@@ -15,8 +15,6 @@ from einops import rearrange, reduce, repeat
 from einops_exts import rearrange_many
 from einops.layers.torch import Rearrange
 
-from dalle2_pytorch.attention import QueryAttnUpsample
-
 # constants
 
 MList = nn.ModuleList
@@ -495,12 +493,10 @@ class ViTEncDec(nn.Module):
                 layers = layers
             ),
             nn.Sequential(
-                nn.Linear(dim, dim * 2, bias = False),
+                nn.Linear(dim, dim * 4, bias = False),
                 nn.Tanh(),
-                nn.Linear(dim * 2, dim, bias = False),
+                nn.Linear(dim * 4, input_dim, bias = False),
             ),
-            nn.LayerNorm(dim),
-            nn.Linear(dim, input_dim),
             RearrangeImage(),
             Rearrange('b h w (p1 p2 c) -> b c (h p1) (w p2)', p1 = patch_size, p2 = patch_size)
         )
